<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="ark" version="18">
<MenuBar>
	<Menu name="archive">
		<text context="Noun, not a verb. Top level menu item, i.e. like File">&amp;Archive</text>
		<Action name="ark_file_new"/>
		<Action name="ark_file_open"/>
		<Action name="ark_file_open_recent"/>
		<Separator/>
		<DefineGroup name="file_save"/>
		<Separator/>
		<DefineGroup name="archive_extract"/>
		<Separator/>
		<DefineGroup name="archive_edit"/>
		<Separator/>
		<DefineGroup name="archive_props"/>
		<Separator/>
		<Action name="ark_quit"/>
	</Menu>
	<Merge/>
	<Menu name="settings">
		<DefineGroup name="settings_show"/>
	</Menu>
	<Menu name="help">
		<Action name="help_welcome_page"/>
	</Menu>
</MenuBar>
</gui>
